//  Differential.h - the differential gear system of the drivetrain.
//
//  Copyright (C) 2001--2002 Sam Varner
//
//  This file is part of Vamos Automotive Simulator.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef _DIFFERENTIAL_H_
#define _DIFFERENTIAL_H_

namespace Vamos_Body
{
  //* The differential gear system of the drivetrain.
  class Differential
  {
	// The differential's gear ratio.
	double m_final_drive;

	// The anti-slip factor.
	double m_anti_slip;

	// The amount of drag produced by the anti-slip mechanism in the
	// last time step.
	double m_last_diff_drag;

	// The angular speed of the left wheel.
	double m_left_wheel_speed;

	// The angular speed of the right wheel.
	double m_right_wheel_speed;

	// The angular speed of the driveshaft.
	double m_driveshaft_speed;

	// The torque to be applied to the left wheel.
	double m_left_torque;

	// The torque to be applied to the right wheel.
	double m_right_torque;

  public:
	//** Constructor
	Differential (double final_drive, double anti_slip);

	// Set the rotational speeds of the wheels.
	void input (double left_wheel_speed, double right_wheel_speed);

	// Set the torque from th driveshaft.
	void torque (double driveshaft_torque);

	// Return the rotational speed of the driveshaft. 
	double driveshaft_speed () const { return m_driveshaft_speed; }

	// Return the torque for the left wheel.
	double left_torque () const { return m_left_torque; }

	// Return the torque for the right wheel.
	double right_torque () const { return m_right_torque; }
  };
}

#endif // !_DIFFERENTIAL_H_
