//	Vamos Automotive Simulator
//  Copyright (C) 2001--2002 Sam Varner
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef _CAR_WORLD_H_
#define _CAR_WORLD_H_

#include <vamos/body/Car.h>
#include <vamos/geometry/Surface.h>
#include <vamos/geometry/Three_Vector.h>
#include <vamos/track/Track.h>
#include <vamos/world/Atmosphere.h>
#include <vamos/world/World.h>

using Vamos_Body::Car;
using Vamos_Track::Road_Segment;

namespace Vamos_World
{
  class Car_Info
  {
	Car* mp_car;
	// A reference to the vector of segments that make up the track.
	const std::vector <Road_Segment*>& m_segments;
	// A vector that contains iterators for the segement for each particle. 
	std::vector <std::vector <Road_Segment*>::const_iterator> 
	m_segment_iterators;
	// A vector that contains the surface for each particle. 
	std::vector <Vamos_Geometry::Surface*> m_surfaces;

	double m_track_distance;
	// timing sector
	int m_sector;
	Three_Vector m_track_coordinates;

  public:
	Car_Info (Car* car, const std::vector <Road_Segment*>& segments);

	Car* car () const { return mp_car; }

	const Road_Segment& segment (int index) const 
	{ return **(m_segment_iterators [index]); }
	const Vamos_Geometry::Surface& surface (int index) const 
	{ return *(m_surfaces [index]); }
	const Vamos_Geometry::Surface& inside_surface () const 
	{ return **(m_surfaces.end () - 1); }
	const Vamos_Geometry::Surface& outside_surface () const 
	{ return **(m_surfaces.begin ()); }

	Three_Vector track_coordinates (const Three_Vector& world_pos, int part);
	Three_Vector center_position ();
	Three_Matrix center_orientation ();

	double distance () const { return m_track_distance; }
	double from_center () const { return m_track_coordinates [1]; }
	double elevation () const { return m_track_coordinates [2]; }
	int sector () const { return m_sector; }
	void sector (int sect) { m_sector = sect; }
	void reset ();
  };

  class Car_World : public World
  {
  protected:
	Car_Info* mp_car_info;
	Car* car ();

  public:
	Car_World (double gravity, Car* car, 
			   Vamos_Track::Track* track, Atmosphere* atmosphere);
	virtual ~Car_World ();
	void interact (Vamos_Body::Body*);
  };
}

#endif
